/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIWorkerDebugger.idl
 */

#ifndef __gen_nsIWorkerDebugger_h__
#define __gen_nsIWorkerDebugger_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIDOMWindow; /* forward declaration */


/* starting interface:    nsIWorkerDebuggerListener */
#define NS_IWORKERDEBUGGERLISTENER_IID_STR "54fd2dd3-c01b-4f71-888f-462f37a54f57"

#define NS_IWORKERDEBUGGERLISTENER_IID \
  {0x54fd2dd3, 0xc01b, 0x4f71, \
    { 0x88, 0x8f, 0x46, 0x2f, 0x37, 0xa5, 0x4f, 0x57 }}

class NS_NO_VTABLE nsIWorkerDebuggerListener : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IWORKERDEBUGGERLISTENER_IID)

  /* void onClose (); */
  NS_IMETHOD OnClose(void) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIWorkerDebuggerListener, NS_IWORKERDEBUGGERLISTENER_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIWORKERDEBUGGERLISTENER \
  NS_IMETHOD OnClose(void) override; 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIWORKERDEBUGGERLISTENER(_to) \
  NS_IMETHOD OnClose(void) override { return _to OnClose(); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIWORKERDEBUGGERLISTENER(_to) \
  NS_IMETHOD OnClose(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->OnClose(); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsWorkerDebuggerListener : public nsIWorkerDebuggerListener
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIWORKERDEBUGGERLISTENER

  nsWorkerDebuggerListener();

private:
  ~nsWorkerDebuggerListener();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsWorkerDebuggerListener, nsIWorkerDebuggerListener)

nsWorkerDebuggerListener::nsWorkerDebuggerListener()
{
  /* member initializers and constructor code */
}

nsWorkerDebuggerListener::~nsWorkerDebuggerListener()
{
  /* destructor code */
}

/* void onClose (); */
NS_IMETHODIMP nsWorkerDebuggerListener::OnClose()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


/* starting interface:    nsIWorkerDebugger */
#define NS_IWORKERDEBUGGER_IID_STR "0833b363-bffe-4cdb-ad50-1c4563e0c8ff"

#define NS_IWORKERDEBUGGER_IID \
  {0x0833b363, 0xbffe, 0x4cdb, \
    { 0xad, 0x50, 0x1c, 0x45, 0x63, 0xe0, 0xc8, 0xff }}

class NS_NO_VTABLE nsIWorkerDebugger : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IWORKERDEBUGGER_IID)

  enum {
    TYPE_DEDICATED = 0U,
    TYPE_SHARED = 1U,
    TYPE_SERVICE = 2U
  };

  /* readonly attribute bool isClosed; */
  NS_IMETHOD GetIsClosed(bool *aIsClosed) = 0;

  /* readonly attribute bool isChrome; */
  NS_IMETHOD GetIsChrome(bool *aIsChrome) = 0;

  /* readonly attribute nsIWorkerDebugger parent; */
  NS_IMETHOD GetParent(nsIWorkerDebugger * *aParent) = 0;

  /* readonly attribute unsigned long type; */
  NS_IMETHOD GetType(uint32_t *aType) = 0;

  /* readonly attribute DOMString url; */
  NS_IMETHOD GetUrl(nsAString & aUrl) = 0;

  /* readonly attribute nsIDOMWindow window; */
  NS_IMETHOD GetWindow(nsIDOMWindow * *aWindow) = 0;

  /* void addListener (in nsIWorkerDebuggerListener listener); */
  NS_IMETHOD AddListener(nsIWorkerDebuggerListener *listener) = 0;

  /* void removeListener (in nsIWorkerDebuggerListener listener); */
  NS_IMETHOD RemoveListener(nsIWorkerDebuggerListener *listener) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIWorkerDebugger, NS_IWORKERDEBUGGER_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIWORKERDEBUGGER \
  NS_IMETHOD GetIsClosed(bool *aIsClosed) override; \
  NS_IMETHOD GetIsChrome(bool *aIsChrome) override; \
  NS_IMETHOD GetParent(nsIWorkerDebugger * *aParent) override; \
  NS_IMETHOD GetType(uint32_t *aType) override; \
  NS_IMETHOD GetUrl(nsAString & aUrl) override; \
  NS_IMETHOD GetWindow(nsIDOMWindow * *aWindow) override; \
  NS_IMETHOD AddListener(nsIWorkerDebuggerListener *listener) override; \
  NS_IMETHOD RemoveListener(nsIWorkerDebuggerListener *listener) override; 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIWORKERDEBUGGER(_to) \
  NS_IMETHOD GetIsClosed(bool *aIsClosed) override { return _to GetIsClosed(aIsClosed); } \
  NS_IMETHOD GetIsChrome(bool *aIsChrome) override { return _to GetIsChrome(aIsChrome); } \
  NS_IMETHOD GetParent(nsIWorkerDebugger * *aParent) override { return _to GetParent(aParent); } \
  NS_IMETHOD GetType(uint32_t *aType) override { return _to GetType(aType); } \
  NS_IMETHOD GetUrl(nsAString & aUrl) override { return _to GetUrl(aUrl); } \
  NS_IMETHOD GetWindow(nsIDOMWindow * *aWindow) override { return _to GetWindow(aWindow); } \
  NS_IMETHOD AddListener(nsIWorkerDebuggerListener *listener) override { return _to AddListener(listener); } \
  NS_IMETHOD RemoveListener(nsIWorkerDebuggerListener *listener) override { return _to RemoveListener(listener); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIWORKERDEBUGGER(_to) \
  NS_IMETHOD GetIsClosed(bool *aIsClosed) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetIsClosed(aIsClosed); } \
  NS_IMETHOD GetIsChrome(bool *aIsChrome) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetIsChrome(aIsChrome); } \
  NS_IMETHOD GetParent(nsIWorkerDebugger * *aParent) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetParent(aParent); } \
  NS_IMETHOD GetType(uint32_t *aType) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetType(aType); } \
  NS_IMETHOD GetUrl(nsAString & aUrl) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetUrl(aUrl); } \
  NS_IMETHOD GetWindow(nsIDOMWindow * *aWindow) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetWindow(aWindow); } \
  NS_IMETHOD AddListener(nsIWorkerDebuggerListener *listener) override { return !_to ? NS_ERROR_NULL_POINTER : _to->AddListener(listener); } \
  NS_IMETHOD RemoveListener(nsIWorkerDebuggerListener *listener) override { return !_to ? NS_ERROR_NULL_POINTER : _to->RemoveListener(listener); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsWorkerDebugger : public nsIWorkerDebugger
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIWORKERDEBUGGER

  nsWorkerDebugger();

private:
  ~nsWorkerDebugger();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsWorkerDebugger, nsIWorkerDebugger)

nsWorkerDebugger::nsWorkerDebugger()
{
  /* member initializers and constructor code */
}

nsWorkerDebugger::~nsWorkerDebugger()
{
  /* destructor code */
}

/* readonly attribute bool isClosed; */
NS_IMETHODIMP nsWorkerDebugger::GetIsClosed(bool *aIsClosed)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute bool isChrome; */
NS_IMETHODIMP nsWorkerDebugger::GetIsChrome(bool *aIsChrome)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsIWorkerDebugger parent; */
NS_IMETHODIMP nsWorkerDebugger::GetParent(nsIWorkerDebugger * *aParent)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute unsigned long type; */
NS_IMETHODIMP nsWorkerDebugger::GetType(uint32_t *aType)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute DOMString url; */
NS_IMETHODIMP nsWorkerDebugger::GetUrl(nsAString & aUrl)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsIDOMWindow window; */
NS_IMETHODIMP nsWorkerDebugger::GetWindow(nsIDOMWindow * *aWindow)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void addListener (in nsIWorkerDebuggerListener listener); */
NS_IMETHODIMP nsWorkerDebugger::AddListener(nsIWorkerDebuggerListener *listener)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void removeListener (in nsIWorkerDebuggerListener listener); */
NS_IMETHODIMP nsWorkerDebugger::RemoveListener(nsIWorkerDebuggerListener *listener)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIWorkerDebugger_h__ */
