/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIContainerBoxObject.idl"

/**
 * @deprecated Please consider using ContainerBoxObject.
 */

[uuid(db436f2f-c656-4754-b0fa-99bc353bd63f)]
interface nsIBrowserBoxObject : nsIContainerBoxObject
{
};

