/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef nsCompartmentInfo_h
#define nsCompartmentInfo_h

#include "nsICompartmentInfo.h"

class nsCompartmentInfo : public nsICompartmentInfo
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSICOMPARTMENTINFO

  nsCompartmentInfo();

private:
  virtual ~nsCompartmentInfo();

protected:
};

#endif
